__author__ = 'dk'
import argparse
from gitdown.parser_git_dir import *
from gitdown import download
import tqdm
import os
import threading


lock = threading.Semaphore(1)
files = []
sem = None
dst_root = os.path.expanduser('~')

def down(repo, file):
    global  sem, lock, files, dst_root

    download.download_file(repo= repo, path= file, dst= dst_root + '/' + file)
    if os.path.exists(file) == False:
        print('{0} download fail! it will be re-download in the future!'.format(file))
        lock.acquire()
        files.append(file)
        lock.release()

    sem.release()


def main():
    global  lock, files, sem, dst_root

    args = argparse.ArgumentParser(description='Download github resposity directory.')
    args.add_argument('--repo', type= str, required= True, help='Something like https://github.com/jmhIcoding/social_webpage')
    args.add_argument('--directory', type=str, required= True, help='Something like pcaps/. Note that do not start with "/" and end with "/", use "" (empty) for root directory.')
    args.add_argument('--dst_root', type=str, required=True, help= 'Indicate where do you want to store the downloaded files and sub-dirs from github.')
    args.add_argument('--thread_num', type=int, default= 5, help='The thread number for download!')
    parsed_args = args.parse_args()

    if parsed_args.directory != '' and (parsed_args.directory[0] == '/' or parsed_args.directory[-1]== '/') :
        raise  ValueError('Oh, directory parameter must not start with "/" and end with "/"!, please refer to the help information.')

    if parsed_args.thread_num < 1:
        raise  ValueError('Oh, thread number should >= 1 ! ')

    dst_root = parsed_args.dst_root
    repo = '/'.join(parsed_args.repo.split('/')[-2:])
    files = parser_git_directory(_repo= repo, directory= parsed_args.directory)

    sem = threading.Semaphore(parsed_args.thread_num)
    threads_list = []
    index = 0
    pbar = tqdm.tqdm(total=len(files))
    while index < len(files):
        pbar.update(1)
        file = files[index]
        index += 1
        if os.path.exists(file) == True:
           print('{0} already download!'.format(file))
           continue

        th = threading.Thread(target=down, args=(repo, file))
        sem.acquire()
        th.start()
        threads_list.append(th)

        if len(files)!= pbar.total:
            pbar.total = len(files)
            pbar.refresh()

if __name__ == '__main__':
    main()






__author__ = 'dk'
import os
from  github import Github
import github

config_path = os.path.expanduser('~') + '/gitdown.token'
if os.path.exists(config_path) == False:
    token = input('Please input your github access token now, first time:')

    try:
        git = Github(login_or_token=token)
        git.get_repo('helloworld9992939329392939294333435452dhfhdfjdfdgfgdfgfd___dfdf')
    except github.GithubException as exp:
        if isinstance(exp, github.BadCredentialsException):
            raise  ValueError('The token you input is invalid!!!')
    with open(config_path,'w') as fp:
        fp.write(token)

else:
    with open(config_path) as fp:
        token = fp.read()

github_access_token= token #github访问的token



import setuptools
__author__ = 'jmh081701'

setuptools.setup(
    name='gitdown',
    version='1.1.3',
    description='A tool to download sub-dirs from github.',

    url='https://github.com/jmhIcoding/GitDown',
    author='jiangminghao',
    author_email='jiangminghao@iie.ac.cn',
    install_requires=[
        'pygithub',
        'requests',
        'tqdm'
    ],
    packages = setuptools.find_packages(),
    entry_points={
        'console_scripts':[
            'gitdown=gitdown.main:main'
        ]
    },
    classifiers=['Programming Language :: Python :: 3']
)


__author__ = 'dk'
from  github import Github
from gitdown.config import github_access_token

def parser_git_directory(_repo, directory):
    git = Github(login_or_token=github_access_token)
    repository = _repo
    repo = git.get_repo(repository)
    directory  =  directory
    dirs= repo.get_dir_contents( '/' + directory)
    filepaths = []
    for file_or_dir in dirs:
            print('Go deep into {0}'.format(directory))
            if file_or_dir.type == 'dir':
                ##说明是文件夹, 递归访问
                sub_url =  directory + '/' + file_or_dir.path.split('/')[-1]
                files = parser_git_directory(_repo, sub_url)
                filepaths += files

            else:
                ##说明是文件, 添加到文件的队列里面去
                filepaths.append(file_or_dir.path)
    return filepaths




__author__ = 'dk'
import os
import sys
import requests
import base64
import json
from gitdown.config import  github_access_token
def download_file(repo, path, dst):
    print('Download {0} now!'.format(path))
    sys.stdout.flush()

    if os.path.exists(os.path.dirname(dst)) == False:
        os.makedirs(os.path.dirname(dst), exist_ok=True)

    url='https://api.github.com/repos/{0}/contents/{1}'.format(repo, path)
    headers = {
        'Accept':'application/vnd.github.v3+json',
        'Authorization':'token {0}'.format(github_access_token)
    }
    get = requests.get(url, headers = headers)
    with open(dst,'w') as fp:
        fp.write(base64.b64decode(get.json()['content']).decode())
    return  get.status_code

if __name__ == '__main__':
    url='https://raw.githubusercontent.com/jmhIcoding/Intra-domain-WPF/master/YouTube/D1_YouTube_compressed/D1_YouTube.z01'
    path = './YouTube/D1_YouTube_compressed/D1_YouTube.z01'
    download_file(url,dst=path)